#!/bin/userenv python
# :mode=python:encoding=ISO-8859-2:
# -*- coding: iso-8859-2 -*-
# Author: 2004 Gulácsi Tamás
#
# Ported from Josh Carter's Perl IPTCInfo.pm by Tam?s Gul?csi
#
# IPTCInfo: extractor for IPTC metadata embedded in images
# Copyright (C) 2000-2004 Josh Carter <josh@multipart-mixed.com>
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the same terms as Python itself.
#
# VERSION = '1.9';
"""
IPTCInfo - Python module for extracting and modifying IPTC image meta-data

Ported from Josh Carter's Perl IPTCInfo-1.9.pm by Tamás Gulácsi

Ever wish you add information to your photos like a caption, the place
you took it, the date, and perhaps even keywords and categories? You
already can. The International Press Telecommunications Council (IPTC)
defines a format for exchanging meta-information in news content, and
that includes photographs. You can embed all kinds of information in
your images. The trick is putting it to use.

That's where this IPTCInfo Python module comes into play. You can embed
information using many programs, including Adobe Photoshop, and
IPTCInfo will let your web server -- and other automated server
programs -- pull it back out. You can use the information directly in
Python programs, export it to XML, or even export SQL statements ready
to be fed into a database.


PREFACE

First, I want to apologize a little bit: as this module is originally
written in Perl by Josh Carter, it is quite non-Pythonic (for example
the addKeyword, clearSupplementalCategories functions - I think it
would be better having a derived list class with add, clear functions)
and tested only by me reading/writing IPTC metadata for family photos.
Any suggestions welcomed!

Thanks,
Tamás Gulácsi <gthomas at fw dot hu>

SYNOPSIS

  from iptcinfo import IPTCInfo
  import sys

  fn = (len(sys.argv) > 1 and [sys.argv[1]] or ['test.jpg'])[0]
  fn2 = (len(sys.argv) > 2 and [sys.argv[2]] or ['test_out.jpg'])[0]

  # Create new info object
  info = IPTCInfo(fn)

  # Check if file had IPTC data
  if len(info.data) < 4: raise Exception(info.error)

  # Print list of keywords, supplemental categories, or contacts
  print info.keywords
  print info.supplementalCategories
  print info.contacts

  # Get specific attributes...
  caption = info.data['caption/abstract']

  # Create object for file that may or may not have IPTC data.
  info = IPTCInfo(fn)

  # Add/change an attribute
  info.data['caption/abstract'] = 'Witty caption here'
  info.data['supplemental category'] = ['portrait']

  # Save new info to file
  ##### See disclaimer in 'SAVING FILES' section #####
  info.save()
  info.saveAs(fn2)

  #re-read IPTC info
  print IPTCInfo(fn2)

DESCRIPTION

  USING IPTCINFO

  To integrate with your own code, simply do something like what's in
  the synopsys above.

  The complete list of possible attributes is given below. These are
  as specified in the IPTC IIM standard, version 4. Keywords and
  categories are handled slightly differently: since these are lists,
  the module allows you to access them as Python lists. Call
  keywords() and supplementalCategories() to get each list.

  IMAGES NOT HAVING IPTC METADATA

  If yout apply

    info = IPTCInfo('file-name-here.jpg')

  to an image not having IPTC metadata, len(info.data) will be 3
  ('supplemental categories', 'keywords', 'contacts') will be empty
  lists.

  MODIFYING IPTC DATA

  You can modify IPTC data in JPEG files and save the file back to
  disk. Here are the commands for doing so:

    # Set a given attribute
    info.data['iptc attribute here'] = 'new value here'

    # Clear the keywords or supp. categories list
    info.keywords = []
    info.supplementalCategories = []
    info.contacts = []

    # Add keywords or supp. categories
    info.keyword.append('frob')

    # You can also add a list reference
    info.keyword.extend(['frob', 'nob', 'widget'])
    info.keyword += ['gadget']

  SAVING FILES

  With JPEG files you can add/change attributes, add keywords, etc., and
  then call:

    info.save()
    info.saveAs('new-file-name.jpg')

  This will save the file with the updated IPTC info. Please only run
  this on *copies* of your images -- not your precious originals! --
  because I'm not liable for any corruption of your images. (If you
  read software license agreements, nobody else is liable,
  either. Make backups of your originals!)

  If you're into image wizardry, there are a couple handy options you
  can use on saving. One feature is to trash the Adobe block of data,
  which contains IPTC info, color settings, Photoshop print settings,
  and stuff like that. The other is to trash all application blocks,
  including stuff like EXIF and FlashPix data. This can be handy for
  reducing file sizes. The options are passed as a dict to save()
  and saveAs(), e.g.:

    info.save({'discardAdobeParts': 'on'})
    info.saveAs('new-file-name.jpg', {'discardAppParts': 'on'})

  Note that if there was IPTC info in the image, or you added some
  yourself, the new image will have an Adobe part with only the IPTC
  information.

  XML AND SQL EXPORT FEATURES

  IPTCInfo also allows you to easily generate XML and SQL from the image
  metadata. For XML, call:

    xml = info.exportXML('entity-name', extra-data,
                         'optional output file name')

  This returns XML containing all image metadata. Attribute names are
  translated into XML tags, making adjustments to spaces and slashes
  for compatibility. (Spaces become underbars, slashes become dashes.)
  You provide an entity name; all data will be contained within this
  entity.  You can optionally provides a reference to a hash of extra
  data. This will get put into the XML, too. (Example: you may want to
  put info on the image's location into the XML.) Keys must be valid
  XML tag names.  You can also provide a filename, and the XML will be
  dumped into there.

  For SQL, it goes like this:

    my mappings = {
         'IPTC dataset name here': 'your table column name here',
         'caption/abstract':       'caption',
         'city':                   'city',
         'province/state':         'state} # etc etc etc.

    statement = info.exportSQL('mytable', mappings, extra-data)

  This returns a SQL statement to insert into your given table name a
  set of values from the image. You pass in a reference to a hash
  which maps IPTC dataset names into column names for the database
  table. As with XML export, you can also provide extra information to
  be stuck into the SQL.

IPTC ATTRIBUTE REFERENCE

  object name               originating program
  edit status               program version
  editorial update          object cycle
  urgency                   by-line
  subject reference         by-line title
  category                  city
  fixture identifier        sub-location
  content location code     province/state
  content location name     country/primary location code
  release date              country/primary location name
  release time              original transmission reference
  expiration date           headline
  expiration time           credit
  special instructions      source
  action advised            copyright notice
  reference service         contact
  reference date            caption/abstract
  reference number          writer/editor
  date created              image type
  time created              image orientation
  digital creation date     language identifier
  digital creation time

  custom1 - custom20: NOT STANDARD but used by Fotostation.
  IPTCInfo also supports these fields.

KNOWN BUGS

IPTC meta-info on MacOS may be stored in the resource fork instead
of the data fork. This program will currently not scan the resource
fork.

I have heard that some programs will embed IPTC info at the end of the
file instead of the beginning. The module will currently only look
near the front of the file. If you have a file with IPTC data that
IPTCInfo can't find, please contact me! I would like to ensure
IPTCInfo works with everyone's files.

AUTHOR

Josh Carter, josh@multipart-mixed.com
"""

__version__ = '1.9.2-rc5'
__author__ = 'Gulácsi, Tamás'

SURELY_WRITE_CHARSET_INFO = False

from struct import pack, unpack
from cStringIO import StringIO
import sys, re, codecs, os

class String(basestring):
  def __iadd__(self, other):
    assert isinstance(other, str)
    super(type(self), self).__iadd__(other)

class EOFException(Exception):
  def __init__(self, *args):
    Exception.__init__(self)
    self._str = '\n'.join(args)

  def __str__(self):
    return self._str

def push(diction, key, value):
  if diction.has_key(key) and isinstance(diction[key], list):
    diction[key].append(value)
  else: diction[key] = value

def duck_typed(obj, prefs):
  if isinstance(prefs, basestring): prefs = [prefs]
  for pref in prefs:
    if not hasattr(obj, pref): return False
  return True

sys_enc = sys.getfilesystemencoding()

# Debug off for production use
debugMode = 0

#####################################
# These names match the codes defined in ITPC's IIM record 2.
# This hash is for non-repeating data items; repeating ones
# are in %listdatasets below.
c_datasets = {
  # 0: 'record version',    # skip -- binary data
  5: 'object name',
  7: 'edit status',
  8: 'editorial update',
  10: 'urgency',
  12: 'subject reference',
  15: 'category',
  20: 'supplemental category',
  22: 'fixture identifier',
  25: 'keywords',
  26: 'content location code',
  27: 'content location name',
  30: 'release date',
  35: 'release time',
  37: 'expiration date',
  38: 'expiration time',
  40: 'special instructions',
  42: 'action advised',
  45: 'reference service',
  47: 'reference date',
  50: 'reference number',
  55: 'date created',
  60: 'time created',
  62: 'digital creation date',
  63: 'digital creation time',
  65: 'originating program',
  70: 'program version',
  75: 'object cycle',
  80: 'by-line',
  85: 'by-line title',
  90: 'city',
  92: 'sub-location',
  95: 'province/state',
  100: 'country/primary location code',
  101: 'country/primary location name',
  103: 'original transmission reference',
  105: 'headline',
  110: 'credit',
  115: 'source',
  116: 'copyright notice',
  118: 'contact',
  120: 'caption/abstract',
  122: 'writer/editor',
#  125: 'rasterized caption', # unsupported (binary data)
  130: 'image type',
  131: 'image orientation',
  135: 'language identifier',
  200: 'custom1', # These are NOT STANDARD, but are used by
  201: 'custom2', # Fotostation. Use at your own risk. They're
  202: 'custom3', # here in case you need to store some special
  203: 'custom4', # stuff, but note that other programs won't
  204: 'custom5', # recognize them and may blow them away if
  205: 'custom6', # you open and re-save the file. (Except with
  206: 'custom7', # Fotostation, of course.)
  207: 'custom8',
  208: 'custom9',
  209: 'custom10',
  210: 'custom11',
  211: 'custom12',
  212: 'custom13',
  213: 'custom14',
  214: 'custom15',
  215: 'custom16',
  216: 'custom17',
  217: 'custom18',
  218: 'custom19',
  219: 'custom20',
}

c_datasets_r = dict([(v, k) for k, v in c_datasets.iteritems()])

class IPTCData(dict):
  """Dict with int/string keys from c_listdatanames"""
  def __init__(self, diction={}, *args, **kwds):
    super(type(self), self).__init__(self, *args, **kwds)
    self.update(dict([(self.keyAsInt(k), v)
                      for k, v in diction.iteritems()]))

  c_cust_pre = 'nonstandard_'
  def keyAsInt(self, key):
    global c_datasets_r
    if isinstance(key, int): return key #and c_datasets.has_key(key): return key
    elif c_datasets_r.has_key(key): return c_datasets_r[key]
    elif (key.startswith(self.c_cust_pre)
          and key[len(self.c_cust_pre):].isdigit()):
      return int(key[len(self.c_cust_pre):])
    else: raise KeyError("Key %s is not in %s!" % (key, c_datasets_r.keys()))

  def keyAsStr(self, key):
    global c_datasets
    if isinstance(key, basestring) and c_datasets_r.has_key(key): return key
    elif c_datasets.has_key(key): return c_datasets[key]
    elif isinstance(key, int): return self.c_cust_pre + str(key)
    else: raise KeyError("Key %s is not in %s!" % (key, c_datasets.keys()))

  def __getitem__(self, name):
    return super(type(self), self).get(self.keyAsInt(name), None)

  def __setitem__(self, name, value):
    key = self.keyAsInt(name)
    o = super(type(self), self)
    if o.has_key(key) and isinstance(o.__getitem__(key), list):
      #print key, c_datasets[key], o.__getitem__(key)
      if isinstance(value, list): o.__setitem__(key, value)
      else: raise ValueError("For %s only lists acceptable!" % name)
    else: o.__setitem__(self.keyAsInt(name), value)

def debug(level, *args):
  if level < debugMode:
    print '\n'.join(map(unicode, args))

def _getSetSomeList(name):
  def getList(self):
    """Returns the list of %s.""" % name
    return self._data[name]

  def setList(self, value):
    """Sets the list of %s.""" % name
    if isinstance(value, (list, tuple)): self._data[name] = list(value)
    elif isinstance(value, basestring):
      self._data[name] = [value]
      print 'Warning: IPTCInfo.%s is a list!' % name
    else: raise ValueError('IPTCInfo.%s is a list!' % name)

  return (getList, setList)


class IPTCInfo(object):
  """info = IPTCInfo('image filename goes here')

  File can be a file-like object or a string. If it is a string, it is
  assumed to be a filename.

  Returns IPTCInfo object filled with metadata from the given image
  file. File on disk will be closed, and changes made to the IPTCInfo
  object will *not* be flushed back to disk.

  If force==True, than forces an object to always be returned. This
  allows you to start adding stuff to files that don't have IPTC info
  and then save it."""

  def __init__(self, fobj, force=False, *args, **kwds):
    # Open file and snarf data from it.
    self.error = None
    self._data = IPTCData({'supplemental category': [], 'keywords': [],
                           'contact': []})
    if duck_typed(fobj, 'read'):
      self._filename = None
      self._fh = fobj
    else:
      self._filename = fobj

    fh = self._getfh()
    self.inp_charset = sys_enc
    self.out_charset = 'utf_8'

    datafound = self.scanToFirstIMMTag(fh)
    if datafound or force:
      # Do the real snarfing here
      if datafound: self.collectIIMInfo(fh)
    else:
      self.log("No IPTC data found.")
      self._closefh(fh)
      raise Exception("No IPTC data found.")
    self._closefh(fh)

  def _closefh(self, fh):
    if fh and self._filename is not None: fh.close()

  def _getfh(self, mode='r'):
    assert self._filename is not None or self._fh is not None
    if self._filename is not None:
      fh = file(self._filename, (mode + 'b').replace('bb', 'b'))
      if not fh:
        self.log("Can't open file")
        return None
      else: return fh
    else: return self._fh

  #######################################################################
  # New, Save, Destroy, Error
  #######################################################################

  def error(self):
    """Returns the last error message"""
    return self.error

  def save(self, options=None):
    """Saves Jpeg with IPTC data back to the same file it came from."""
    assert self._filename is not None
    return self.saveAs(self._filename, options)

  def _filepos(self, fh):
    fh.flush()
    return 'POS=%d\n' % fh.tell()

  def saveAs(self, newfile, options=None):
    """Saves Jpeg with IPTC data to a given file name."""
    assert self._filename is not None
    # Open file and snarf data from it.
    fh = self._getfh()
    if not self.fileIsJpeg(fh):
      self.log("Source file is not a Jpeg; I can only save Jpegs. Sorry.")
      return None
    ret = self.jpegCollectFileParts(fh, options)
    self._closefh(fh)
    if ret is None:
      self.log("collectfileparts failed")
      raise Exception('collectfileparts failed')

    (start, end, adobe) = ret
    debug(2, 'start: %d, end: %d, adobe:%d' % tuple(map(len, ret)))
    self.hexDump(start), len(end)
    debug(3, 'adobe1', adobe)
    if options is not None and options.has_key('discardAdobeParts'):
      adobe = None
    debug(3, 'adobe2', adobe)

    debug(1, 'writing...')
    # fh = os.tmpfile() ## 20051011 - Windows doesn't like tmpfile ##
    # Open dest file and stuff data there
    # fh.truncate()
    # fh.seek(0, 0)
    # debug(2, self._filepos(fh))
    fh = StringIO()
    if not fh:
      self.log("Can't open output file")
      return None
    debug(3, len(start), len(end))
    fh.write(start)
    # character set
    ch = self.c_charset_r.get((self.out_charset is None and [self.inp_charset]
                               or [self.out_charset])[0], None)
    # writing the character set is not the best practice - couldn't find the needed place (record) for it yet!
    if SURELY_WRITE_CHARSET_INFO and ch is not None:
      fh.write(pack("!BBBHH", 0x1c, 1, 90, 4, ch))


    debug(2, self._filepos(fh))
    #$self->PhotoshopIIMBlock($adobe, $self->PackedIIMData());
    data = self.photoshopIIMBlock(adobe, self.packedIIMData())
    debug(3, len(data), self.hexDump(data))
    fh.write(data)
    debug(2, self._filepos(fh))
    fh.flush()
    fh.write(end)
    debug(2, self._filepos(fh))
    fh.flush()

    #copy the successfully written file back to the given file
    fh2 = file(newfile, 'wb')
    fh2.truncate()
    fh2.seek(0,0)
    fh.seek(0, 0)
    while 1:
      buf = fh.read(8192)
      if buf is None or len(buf) == 0: break
      fh2.write(buf)
    self._closefh(fh)
    fh2.flush()
    fh2.close()
    return True

  def __destroy__(self):
    """Called when object is destroyed. No action necessary in this case."""
    pass


  #######################################################################
  # Attributes for clients
  #######################################################################

  def getData(self):
    return self._data
  def setData(self, value):
    raise Exception('You cannot overwrite the data, only its elements!')
  data = property(getData, setData)

  keywords = property(*_getSetSomeList('keywords'))
  supplementalCategories = property(*_getSetSomeList('supplemental category'))
  contacts = property(*_getSetSomeList('contact'))

  def __str__(self):
    return ('charset: ' + self.inp_charset + '\n'
            + str(dict([(self._data.keyAsStr(k), v)
                       for k, v in self._data.iteritems()])))


  def readExactly(self, fh, length):
    """readExactly

    Reads exactly length bytes and throws an exception if EOF is hitten before.
    """
    ## assert isinstance(fh, file)
    assert duck_typed(fh, 'read') # duck typing
    buf = fh.read(length)
    if buf is None or len(buf) < length: raise EOFException('readExactly: %s' % str(fh))
    return buf

  def seekExactly(self, fh, length):
    """seekExactly

    Seeks length bytes from the current position and checks the result
    """
    ## assert isinstance(fh, file)
    assert duck_typed(fh, ['seek', 'tell']) # duck typing
    pos = fh.tell()
    fh.seek(length, 1)
    if fh.tell() - pos != length: raise EOFException()


  #######################################################################
  # XML, SQL export
  #######################################################################

  def exportXML(self, basetag, extra, filename):
    """xml = info.exportXML('entity-name', extra-data,
                            'optional output file name')

    Exports XML containing all image metadata. Attribute names are
    translated into XML tags, making adjustments to spaces and slashes
    for compatibility. (Spaces become underbars, slashes become
    dashes.)  Caller provides an entity name; all data will be
    contained within this entity. Caller optionally provides a
    reference to a hash of extra data. This will be output into the
    XML, too. Keys must be valid XML tag names. Optionally provide a
    filename, and the XML will be dumped into there."""

    P = lambda s: '  '*off + s + '\n'
    off = 0

    if len(basetag) == 0: basetag = 'photo'
    out = P("<%s>" % basetag)

    off += 1
    # dump extra info first, if any
    for k, v in (isinstance(extra, dict) and [extra] or [{}])[0].iteritems():
      out += P("<%s>%s</%s>" % (k, v, k))

    # dump our stuff
    for k, v in self._data.iteritems():
      if not isinstance(v, list):
        key = re.sub('/', '-', re.sub(' +', ' ', self._data.keyAsStr(k)))
        out += P("<%s>%s</%s>" % (key, v, key))

    # print keywords
    kw = self.keywords()
    if kw and len(kw) > 0:
      out += P("<keywords>")
      off += 1
      for k in kw: out += P("<keyword>%s</keyword>" % k)
      off -= 1
      out += P("</keywords>")

    # print supplemental categories
    sc = self.supplementalCategories()
    if sc and len(sc) > 0:
      out += P("<supplemental_categories>")
      off += 1
      for k in sc:
        out += P("<supplemental_category>%s</supplemental_category>" % k)
      off -= 1
      out += P("</supplemental_categories>")

    # print contacts
    kw = self.contacts()
    if kw and len(kw) > 0:
      out += P("<contacts>")
      off += 1
      for k in kw: out += P("<contact>%s</contact>" % k)
      off -= 1
      out += P("</contacts>")

    # close base tag
    off -= 1
    out += P("</%s>" % basetag)

    # export to file if caller asked for it.
    if len(filename) > 0:
      xmlout = file(filename, 'wb')
      xmlout.write(out)
      xmlout.close()

    return out

  def exportSQL(self, tablename, mappings, extra):
    """statement = info.exportSQL('mytable', mappings, extra-data)

    mappings = {
      'IPTC dataset name here': 'your table column name here',
      'caption/abstract': 'caption',
      'city': 'city',
      'province/state': 'state} # etc etc etc.

    Returns a SQL statement to insert into your given table name a set
    of values from the image. Caller passes in a reference to a hash
    which maps IPTC dataset names into column names for the database
    table. Optionally pass in a ref to a hash of extra data which will
    also be included in the insert statement. Keys in that hash must
    be valid column names."""

    if (tablename is None or mappings is None): return None
    statement = columns = values = None

    E = lambda s: "'%s'" % re.sub("'", "''", s) # escape single quotes

    # start with extra data, if any
    columns = ', '.join(extra.keys() + mappings.keys())
    values = ', '.join(map(E, extra.values()
                           + [self.getdata(k) for k in mappings.keys()]))
    # process our data

    statement = "INSERT INTO %s (%s) VALUES (%s)" \
                % (tablename, columns, values)

    return statement

  #######################################################################
  # File parsing functions (private)
  #######################################################################

  def scanToFirstIMMTag(self, fh): #OK#
    """Scans to first IIM Record 2 tag in the file. The will either
    use smart scanning for Jpegs or blind scanning for other file
    types."""
    ## assert isinstance(fh, file)
    if self.fileIsJpeg(fh):
      self.log("File is Jpeg, proceeding with JpegScan")
      return self.jpegScan(fh)
    else:
      self.log("File not a JPEG, trying BlindScan")
      return self.blindScan(fh)

  def fileIsJpeg(self, fh): #OK#
    """Checks to see if this file is a Jpeg/JFIF or not. Will reset
    the file position back to 0 after it's done in either case."""

    # reset to beginning just in case
    ## assert isinstance(fh, file)
    assert duck_typed(fh, ['read', 'seek'])
    fh.seek(0, 0)
    if debugMode > 0:
      self.log("Opening 16 bytes of file:\n");
      dump = fh.read(16)
      debug(3, self.hexDump(dump))
      fh.seek(0, 0)
    # check start of file marker
    ered = False
    try:
      (ff, soi) = fh.read(2)
      if not (ord(ff) == 0xff and ord(soi) == 0xd8): ered = False
      else:
        # now check for APP0 marker. I'll assume that anything with a SOI
        # followed by APP0 is "close enough" for our purposes. (We're not
        # dinking with image data, so anything following the Jpeg tagging
        # system should work.)
        (ff, app0) = fh.read(2)
        if not (ord(ff) == 0xff): ered = False
        else: ered = True
    finally:
      # reset to beginning of file
      fh.seek(0, 0)
      return ered

  c_marker_err = {0: "Marker scan failed",
                  0xd9:  "Marker scan hit end of image marker",
                  0xda: "Marker scan hit start of image data"}
  def jpegScan(self, fh): #OK#
    """Assuming the file is a Jpeg (see above), this will scan through
    the markers looking for the APP13 marker, where IPTC/IIM data
    should be found. While this isn't a formally defined standard, all
    programs have (supposedly) adopted Adobe's technique of putting
    the data in APP13."""
    # Skip past start of file marker
    ## assert isinstance(fh, file)
    try: (ff, soi) = self.readExactly(fh, 2)
    except EOFException: return None

    if not (ord(ff) == 0xff and ord(soi) == 0xd8):
      self.error = "JpegScan: invalid start of file"
      self.log(self.error)
      return None
    # Scan for the APP13 marker which will contain our IPTC info (I hope).
    while 1:
      err = None
      marker = self.jpegNextMarker(fh)
      if ord(marker) == 0xed: break #237

      err = self.c_marker_err.get(ord(marker), None)
      if err is None and self.jpegSkipVariable(fh) == 0:
        err = "JpegSkipVariable failed"
      if err is not None:
        self.error = err
        self.log(err)
        return None

    # If were's here, we must have found the right marker. Now
    # BlindScan through the data.
    return self.blindScan(fh, MAX=self.jpegGetVariableLength(fh))

  def jpegNextMarker(self, fh): #OK#
    """Scans to the start of the next valid-looking marker. Return
    value is the marker id."""

    ## assert isinstance(fh, file)
    # Find 0xff byte. We should already be on it.
    try: byte = self.readExactly(fh, 1)
    except EOFException: return None

    while ord(byte) != 0xff:
      self.log("JpegNextMarker: warning: bogus stuff in Jpeg file");
      try: byte = self.readExactly(fh, 1)
      except EOFException: return None
    # Now skip any extra 0xffs, which are valid padding.
    while 1:
      try: byte = self.readExactly(fh, 1)
      except EOFException: return None
      if ord(byte) != 0xff: break

    # byte should now contain the marker id.
    self.log("JpegNextMarker: at marker %02X (%d)" % (ord(byte), ord(byte)))
    return byte

  def jpegGetVariableLength(self, fh): #OK#
    """Gets length of current variable-length section. File position
    at start must be on the marker itself, e.g. immediately after call
    to JPEGNextMarker. File position is updated to just past the
    length field."""
    ## assert isinstance(fh, file)
    try: length = unpack('!H', self.readExactly(fh, 2))[0]
    except EOFException: return 0
    self.log('JPEG variable length: %d' % length)

    # Length includes itself, so must be at least 2
    if length < 2:
      self.log("JPEGGetVariableLength: erroneous JPEG marker length")
      return 0
    return length-2

  def jpegSkipVariable(self, fh, rSave=None): #OK#
    """Skips variable-length section of Jpeg block. Should always be
    called between calls to JpegNextMarker to ensure JpegNextMarker is
    at the start of data it can properly parse."""

    ## assert isinstance(fh, file)
    # Get the marker parameter length count
    length = self.jpegGetVariableLength(fh)
    if length == 0: return None

    # Skip remaining bytes
    if rSave is not None or debugMode > 0:
      try: temp = self.readExactly(fh, length)
      except EOFException:
        self.log("JpegSkipVariable: read failed while skipping var data");
        return None
    # prints out a heck of a lot of stuff
    # self.hexDump(temp)
    else:
      # Just seek
      try: self.seekExactly(fh, length)
      except EOFException:
        self.log("JpegSkipVariable: read failed while skipping var data");
        return None

    return (rSave is not None and [temp] or [True])[0]

  c_charset = {100: 'iso8859_1', 101: 'iso8859_2', 109: 'iso8859_3',
               110: 'iso8859_4', 111: 'iso8859_5', 125: 'iso8859_7',
               127: 'iso8859_6', 138: 'iso8859_8',
               196: 'utf_8'}
  c_charset_r = dict([(v, k) for k, v in c_charset.iteritems()])
  def blindScan(self, fh, MAX=8192): #OK#
    """Scans blindly to first IIM Record 2 tag in the file. This
    method may or may not work on any arbitrary file type, but it
    doesn't hurt to check. We expect to see this tag within the first
    8k of data. (This limit may need to be changed or eliminated
    depending on how other programs choose to store IIM.)"""

    ## assert isinstance(fh, file)
    assert duck_typed(fh, 'read')
    offset = 0
    # keep within first 8192 bytes
    # NOTE: this may need to change
    self.log('blindScan: starting scan, max length %d' % MAX)

    # start digging
    while offset <= MAX:
      try: temp = self.readExactly(fh, 1)
      except EOFException:
        self.log("BlindScan: hit EOF while scanning");
        return None
      # look for tag identifier 0x1c
      if ord(temp) == 0x1c:
        # if we found that, look for record 2, dataset 0
        # (record version number)
        (record, dataset) = fh.read(2)
        if ord(record) == 1 and ord(dataset) == 90:
          # found character set's record!
          try:
            temp = self.readExactly(fh, self.jpegGetVariableLength(fh))
            self.inp_charset = self.c_charset.get(unpack('!H', temp)[0],
                                                  sys_enc)
            self.log("BlindScan: found character set '%s' at offset %d"
                     % (self.inp_charset, offset))
          except EOFException:
            pass

        elif ord(record) == 2:
          # found it. seek to start of this tag and return.
          self.log("BlindScan: found IIM start at offset %d" % offset);
          try: self.seekExactly(fh, -3) # seek rel to current position
          except EOFException:
            return None
          return offset
        else:
          # didn't find it. back up 2 to make up for
          # those reads above.
          try: self.seekExactly(fh, -2) # seek rel to current position
          except EOFException: return None

      # no tag, keep scanning
      offset += 1

    return False

  def collectIIMInfo(self, fh): #OK#
    """Assuming file is seeked to start of IIM data (using above),
    this reads all the data into our object's hashes"""
    # NOTE: file should already be at the start of the first
    # IPTC code: record 2, dataset 0.
    ## assert isinstance(fh, file)
    assert duck_typed(fh, 'read')
    while 1:
      try: header = self.readExactly(fh, 5)
      except EOFException: return None

      (tag, record, dataset, length) = unpack("!BBBH", header)
      # bail if we're past end of IIM record 2 data
      if not (tag == 0x1c and record == 2): return None

      alist = {'tag': tag, 'record': record, 'dataset': dataset,
               'length': length}
      debug(1, '\n'.join(['%s\t: %s' % (k, v) for k, v in alist.iteritems()]))
      value = fh.read(length)

      try: value = unicode(value, encoding=self.inp_charset, errors='strict')
      except:
        self.log('Data "%s" is not in encoding %s!' % (value, self.inp_charset))
        value = unicode(value, encoding=self.inp_charset, errors='replace')

      # try to extract first into _listdata (keywords, categories)
      # and, if unsuccessful, into _data. Tags which are not in the
      # current IIM spec (version 4) are currently discarded.
      if self._data.has_key(dataset) and isinstance(self._data[dataset], list):
        self._data[dataset] += [value]
      elif dataset != 0:
        self._data[dataset] = value

  #######################################################################
  # File Saving
  #######################################################################

  def jpegCollectFileParts(self, fh, discardAppParts=False):
    """Collects all pieces of the file except for the IPTC info that
    we'll replace when saving. Returns the stuff before the info,
    stuff after, and the contents of the Adobe Resource Block that the
    IPTC data goes in. Returns None if a file parsing error occured."""

    ## assert isinstance(fh, file)
    assert duck_typed(fh, ['seek', 'read'])
    adobeParts = ''
    start = ''

    # Start at beginning of file
    fh.seek(0, 0)
    # Skip past start of file marker
    (ff, soi) = fh.read(2)
    if not (ord(ff) == 0xff and ord(soi) == 0xd8):
      self.error = "JpegScan: invalid start of file"
      self.log(self.error)
      return None

    # Begin building start of file
    start += pack("BB", 0xff, 0xd8)

    # Get first marker in file. This will be APP0 for JFIF or APP1 for
    # EXIF.
    marker = self.jpegNextMarker(fh)
    app0data = ''
    app0data = self.jpegSkipVariable(fh, app0data)
    if app0data is None:
      self.error = 'jpegSkipVariable failed'
      self.log(error)
      return None

    if ord(marker) == 0xe0 or not discardAppParts:
      # Always include APP0 marker at start if it's present.
      start += pack('BB', 0xff, ord(marker))
      # Remember that the length must include itself (2 bytes)
      start += pack('!H', len(app0data)+2)
      start += app0data
    else:
      # Manually insert APP0 if we're trashing application parts, since
      # all JFIF format images should start with the version block.
      debug(2, 'discardAppParts=', discardAppParts)
      start += pack("BB", 0xff, 0xe0)
      start += pack("!H", 16)    # length (including these 2 bytes)
      start += "JFIF"            # format
      start += pack("BB", 1, 2)  # call it version 1.2 (current JFIF)
      start += pack('8B', 0)     # zero everything else

    # Now scan through all markers in file until we hit image data or
    # IPTC stuff.
    end = ''
    while 1:
      marker = self.jpegNextMarker(fh)
      if marker is None or ord(marker) == 0:
        self.error = "Marker scan failed"
        self.log(self.error)
        return None
      # Check for end of image
      elif ord(marker) == 0xd9:
        self.log("JpegCollectFileParts: saw end of image marker")
        end += pack("BB", 0xff, ord(marker))
        break
      # Check for start of compressed data
      elif ord(marker) == 0xda:
        self.log("JpegCollectFileParts: saw start of compressed data")
        end += pack("BB", 0xff, ord(marker))
        break
      partdata = ''
      partdata = self.jpegSkipVariable(fh, partdata)
      if not partdata:
        self.error = "JpegSkipVariable failed"
        self.log(self.error)
        return None
      partdata = str(partdata)

      # Take all parts aside from APP13, which we'll replace
      # ourselves.
      if (discardAppParts and ord(marker) >= 0xe0 and ord(marker) <= 0xef):
        # Skip all application markers, including Adobe parts
        adobeParts = ''
      elif ord(marker) == 0xed:
        # Collect the adobe stuff from part 13
        adobeParts = self.collectAdobeParts(partdata)
        break
      else:
        # Append all other parts to start section
        start += pack("BB", 0xff, ord(marker))
        start += pack("!H", len(partdata) + 2)
        start += partdata

    # Append rest of file to end
    while 1:
      buff = fh.read()
      if buff is None or len(buff) == 0: break
      end += buff

    return (start, end, adobeParts)

  def collectAdobeParts(self, data):
    """Part APP13 contains yet another markup format, one defined by
    Adobe.  See"File Formats Specification" in the Photoshop SDK
    (avail from www.adobe.com). We must take
    everything but the IPTC data so that way we can write the file back
    without losing everything else Photoshop stuffed into the APP13
    block."""
    assert isinstance(data, basestring)
    length = len(data)
    offset = 0
    out = ''
    # Skip preamble
    offset = len('Photoshop 3.0 ')
    # Process everything
    while offset < length:
      # Get OSType and ID
      (ostype, id1, id2) = unpack("!LBB", data[offset:offset+6])
      offset += 6

      # Get pascal string
      stringlen = unpack("B", data[offset:offset+1])[0]
      offset += 1
      string = data[offset:offset+stringlen]
      offset += stringlen

      # round up if odd
      if (stringlen % 2 != 0): offset += 1
      # there should be a null if string len is 0
      if stringlen == 0: offset += 1

      # Get variable-size data
      size = unpack("!L", data[offset:offset+4])[0]
      offset += 4

      var = data[offset:offset+size]
      offset += size
      if size % 2 != 0: offset += 1 # round up if odd

      # skip IIM data (0x0404), but write everything else out
      if not (id1 == 4 and id2 == 4):
        out += pack("!LBB", ostype, id1, id2)
        out += pack("B", stringlen)
        out += string
        if stringlen == 0 or stringlen % 2 != 0: out += pack("B", 0)
        out += pack("!L", size)
        out += var
        if size % 2 != 0 and len(out) % 2 != 0: out += pack("B", 0)

    return out

  def _enc(self, text):
    """Recodes the given text from the old character set to utf-8"""
    res = text
    out_charset = (self.out_charset is None and [self.inp_charset]
                   or [self.out_charset])[0]
    if isinstance(text, unicode): res = text.encode(out_charset)
    elif isinstance(text, str):
      try: res = unicode(text, encoding=self.inp_charset).encode(out_charset)
      except:
        self.log("_enc: charset %s is not working for %s"
                 % (self.inp_charset, text))
        res = unicode(text, encoding=self.inp_charset, errors='replace'
                      ).encode(out_charset)
    elif isinstance(text, (list, tuple)):
      res = type(text)(map(self._enc, text))
    return res

  def packedIIMData(self):
    """Assembles and returns our _data and _listdata into IIM format for
    embedding into an image."""
    out = ''
    (tag, record) = (0x1c, 0x02)
    # Print record version
    # tag - record - dataset - len (short) - 4 (short)
    out += pack("!BBBHH", tag, record, 0, 2, 4)

    debug(3, self.hexDump(out))
    # Iterate over data sets
    for dataset, value in self._data.iteritems():
      if len(value) == 0: continue
      if not (c_datasets.has_key(dataset) or isinstance(dataset, int)):
        self.log("PackedIIMData: illegal dataname '%s' (%d)"
                 % (c_datasets[dataset], dataset))
        continue
      value = self._enc(value)
      print value
      if not isinstance(value, list):
        value = str(value)
        out += pack("!BBBH", tag, record, dataset, len(value))
        out += value
      else:
        for v in map(str, value):
          out += pack("!BBBH", tag, record, dataset, len(v))
          out += v

    return out

  def photoshopIIMBlock(self, otherparts, data):
    """Assembles the blob of Photoshop "resource data" that includes our
    fresh IIM data (from PackedIIMData) and the other Adobe parts we
    found in the file, if there were any."""
    out = ''
    assert isinstance(data, basestring)
    resourceBlock = "Photoshop 3.0"
    resourceBlock += pack("B", 0)
    # Photoshop identifier
    resourceBlock += "8BIM"
    # 0x0404 is IIM data, 00 is required empty string
    resourceBlock += pack("BBBB", 0x04, 0x04, 0, 0)
    # length of data as 32-bit, network-byte order
    resourceBlock += pack("!L", len(data))
    # Now tack data on there
    resourceBlock += data
    # Pad with a blank if not even size
    if len(data) % 2 != 0: resourceBlock += pack("B", 0)
    # Finally tack on other data
    if otherparts is not None: resourceBlock += otherparts

    out += pack("BB", 0xff, 0xed) # Jpeg start of block, APP13
    out += pack("!H", len(resourceBlock) + 2) # length
    out += resourceBlock

    return out

  #######################################################################
  # Helpers, docs
  #######################################################################

  def log(self, string):
    """log: just prints a message to STDERR if debugMode is on."""
    if debugMode > 0:
      sys.stderr.write("**IPTC** %s\n" % string)

  def hexDump(self, dump):
    """Very helpful when debugging."""
    length  = len(dump)
    P = lambda z: ((ord(z) >= 0x21 and ord(z) <= 0x7e) and [z] or ['.'])[0]
    ROWLEN = 18
    ered = '\n'
    for j in range(0, length/ROWLEN + int(length%ROWLEN>0)):
      row = dump[j*ROWLEN:(j+1)*ROWLEN]
      ered += ('%02X '*len(row) + '   '*(ROWLEN-len(row)) + '| %s\n') % \
              tuple(map(ord, row) + [''.join(map(P, row))])
    return ered

  def jpegDebugScan(filename):
    """Also very helpful when debugging."""
    assert isinstance(filename, basestring) and os.path.isfile(filename)
    fh = file(filename, 'wb')
    if not fh: raise Exception("Can't open %s" % filename)

    # Skip past start of file marker
    (ff, soi) = fh.read(2)
    if not (ord(ff) == 0xff and ord(soi) == 0xd8):
      self.log("JpegScan: invalid start of file")
    else:
      # scan to 0xDA (start of scan), dumping the markers we see between
      # here and there.
      while 1:
        marker = self.jpegNextMarker(fh)
        if ord(marker) == 0xda: break

        if ord(marker) == 0:
          self.log("Marker scan failed")
          break
        elif ord(marker) == 0xd9:
          self.log("Marker scan hit end of image marker")
          break

        if not self.jpegSkipVariable(fh):
          self.log("JpegSkipVariable failed")
          return None

    self._closefh(fh)

if __name__ == '__main__':
  if len(sys.argv) > 1:
    info = IPTCInfo(sys.argv[1])
    print info



from zope import interface
from zope import component
from p4a.image import interfaces

import p4a.z2utils #Patch CMFDynamicViewFTI
p4a.z2utils # pyflakes

from Products.CMFDynamicViewFTI import interfaces as cmfdynifaces

class ImageContainerDynamicViews(object):
    
    interface.implements(cmfdynifaces.IDynamicallyViewable)
    component.adapts(interfaces.IImageContainerEnhanced)

    def __init__(self, context):
        self.context = context # Actually ignored...
        
    def getAvailableViewMethods(self):
        """Get a list of registered view method names
        """
        return [view for view, name in self.getAvailableLayouts()]

    def getDefaultViewMethod(self):
        """Get the default view method name
        """
        return "image-container.html"

    def getAvailableLayouts(self):
        """Get the layouts registered for this object.
        """        
        return (("image-container.html", "Image view"),)



from zope import interface
from p4a.image import interfaces
from p4a.subtyper.interfaces import (IPortalTypedDescriptor,
                                     IPortalTypedFolderishDescriptor)

class ImageDescriptor(object):
    interface.implements(IPortalTypedDescriptor)

    title = u'Image'
    description = u'Image-based media content'
    type_interface = interfaces.IImageEnhanced
    for_portal_type = 'File'

class AbstractImageContainerDescriptor(object):
    interface.implements(IPortalTypedFolderishDescriptor)

    title = u'Image Container'
    description = u'Container for holding Image-based media content'
    type_interface = interfaces.IImageContainerEnhanced

class FolderImageContainerDescriptor(AbstractImageContainerDescriptor):
    for_portal_type = 'Folder'

class BTreeFolderImageContainerDescriptor(AbstractImageContainerDescriptor):
    for_portal_type = 'LargePloneFolder'

class TopicImageContainerDescriptor(AbstractImageContainerDescriptor):
    for_portal_type = 'Topic'



from zope.schema import interfaces as schema_ifaces
from zope.app.form import interfaces as za_form_ifaces

from p4a.image import interfaces

class ValidationView(object):

    interface = interfaces.IImage

    def __call__(self):
        """Validate the adapted context against the interface"""
        adapter = self.interface(self.context)
        errors = []
        for name in self.interface:
            field = self.interface[name]
            field.bind(adapter)
            try:
                field.validate(getattr(adapter, name))
            except schema_ifaces.ValidationError, error:
                error = za_form_ifaces.WidgetInputError(
                    name, field.title, error)
                errors.append(error)
        return errors



from persistent.dict import PersistentDict
from zope import event
from zope import interface
from zope.app.annotation import interfaces as annointerfaces
from zope.app.event import objectevent
from p4a.image import interfaces
from p4a.fileimage import DictProperty

class ImageAnnotationAddedEvent(objectevent.ObjectEvent):
    """Annotations added to an object for image metadata.
    """

class AnnotationImage(object):
    """An IImage adapter designed to handle ATCT based content.
    """
    
    interface.implements(interfaces.IImage)

    ANNO_KEY = 'p4a.image.imageanno.AnnotationImage'

    def __init__(self, context):
        self.context = context
        annotations = annointerfaces.IAnnotations(context)
        self.image_data = annotations.get(self.ANNO_KEY, None)
        if self.image_data is None:
            self.image_data = PersistentDict()
            annotations[self.ANNO_KEY] = self.image_data
            event.notify(ImageAnnotationAddedEvent(self))

    title           = DictProperty(interfaces.IImage['title'],          'image_data')
    description     = DictProperty(interfaces.IImage['description'],    'image_data')
    photographer    = DictProperty(interfaces.IImage['photographer'],   'image_data')
    dateCreated     = DictProperty(interfaces.IImage['dateCreated'],    'image_data')
    width           = DictProperty(interfaces.IImage['width'],          'image_data')
    height          = DictProperty(interfaces.IImage['height'],         'image_data')
    copyright       = DictProperty(interfaces.IImage['copyright'],      'image_data')
    keywords        = DictProperty(interfaces.IImage['keywords'],       'image_data')
    location        = DictProperty(interfaces.IImage['location'],       'image_data')
    city            = DictProperty(interfaces.IImage['city'],           'image_data')
    state           = DictProperty(interfaces.IImage['state'],          'image_data')
    country         = DictProperty(interfaces.IImage['country'],        'image_data')
    resUnit         = DictProperty(interfaces.IImage['resUnit'],        'image_data')
    xRes            = DictProperty(interfaces.IImage['xRes'],           'image_data')
    yRes            = DictProperty(interfaces.IImage['yRes'],           'image_data')
    comment     = DictProperty(interfaces.IImage['comment'],    'image_data')
    cameraModel     = DictProperty(interfaces.IImage['cameraModel'],    'image_data')
    iso             = DictProperty(interfaces.IImage['iso'],            'image_data')
    file            = DictProperty(interfaces.IImage['file'],           'image_data')
    image_thumbnail = DictProperty(interfaces.IImage['image_thumbnail'],'image_data')
    image_type      = DictProperty(interfaces.IImage['image_type'],     'image_data')
    gpsLat          = DictProperty(interfaces.IImage['gpsLat'],         'image_data')
    gpsLong         = DictProperty(interfaces.IImage['gpsLong'],        'image_data')
    gpsAlt          = DictProperty(interfaces.IImage['gpsAlt'],         'image_data')


class AnnotationImageContainer(object):
    """
    An IImageContainer adapter designed to handle ATCT based image content.
    """

    interface.implements(interfaces.IImageContainer)

    ANNO_KEY = 'p4a.image.imageanno.AnnotationImageContainer'

    def __init__(self, context):
        self.context = context
        annotations = annointerfaces.IAnnotations(context)
        self.image_data = annotations.get(self.ANNO_KEY, None)
        if self.image_data is None:
            self.image_data = PersistentDict()
            annotations[self.ANNO_KEY] = self.image_data
            event.notify(ImageAnnotationAddedEvent(self))

    def set_title(self,v):
        self.context.setTitle(v)
    def get_title(self):
        return self.context.Title()
    title = property(get_title, set_title)

    def set_description(self,v):
        self.context.setDescription(v)
    def get_description(self):
        return self.context.Description()
    description = property(get_description, set_description)

    photographer = DictProperty(interfaces.IImageContainer['photographer'], 'image_data')
    copyright = DictProperty(
        interfaces.IImageContainer['copyright'], 'image_data')
    dateCreated = DictProperty(interfaces.IImageContainer['dateCreated'], 'image_data')
    #genre = DictProperty(interfaces.IImageContainer['genre'], 'image_data')
    folder_image = DictProperty(interfaces.IImageContainer['folder_image'], 'image_data')



from zope import interface
from zope import component
from p4a import subtyper
from p4a.image import interfaces

class MediaActivator(object):
    """An adapter for seeing the activation status or toggling activation.
    """

    interface.implements(interfaces.IMediaActivator)
    component.adapts(interface.Interface)

    def __init__(self, context):
        self.context = context
  
    media_activated = subtyper.activated('p4a.image.Image', 'context')



import mimetypes
import os
import logging

from PIL import Image

from zope.app.annotation import interfaces as annointerfaces
from zope import interface
from zope import datetime

from OFS import Image as ofsimage

from p4a.image import interfaces
from p4a.image.thirdparty import EXIF
from p4a.image.thirdparty import iptcinfo
from p4a.fileimage import utils as fileutils

marker = object()

logger = logging.getLogger('p4a.image')

# XXX fix this
def write_image_thumbnail(id3tags, image_thumbnail):
    size = image_thumbnail.get_size()
    mime_type = image_thumbnail.content_type
    desc = u''

    tempfilename = fileutils.write_to_tempfile(image_thumbnail)
    frame = frames.ImageFrame.create(frames.ImageFrame.FRONT_COVER, 
                                     tempfilename,
                                     desc)

    imgs = id3tags.getImages()
    if len(imgs) == 0:
        id3tags.frames.append(frame)
    else:
        # find the frame index of the first image so we can
        # replace it with our new image frame
        for i in id3tags.frames:
            if i == imgs[0]:
                index = id3tags.frames.index(i)
                id3tags.frames[index] = frame
                break

class ImageDataAccessor(object):
    interface.implements(interfaces.IImageDataAccessor)
    
    def __init__(self, context):
        self._filecontent = context

    @property
    def image_type(self):
        # Use the mime type?
        return self._filecontent.getContentType()
        #return 'JPEG'

    @property
    def _image(self):
        image = getattr(self, '__cached_image', None)
        if image is not None:
            return image
        self.__cached_image = interfaces.IImage(self._filecontent)
        return self.__cached_image

    @property
    def _image_data(self):
        annotations = annointerfaces.IAnnotations(self._filecontent)
        return annotations.get(self._image.ANNO_KEY, None)


    def _getValue(self, metadata_dict, key_name, default=''):
        '''
        Receive dictionary and key name and return value. If no 
        value, return default.
        '''
        try:
            return metadata_dict[key_name]
        except:
            return default


    def _getInfo(self, filename):
        '''
        Get metadata that all images have.  This includes mime type (from Zope) and
        the dimensions of the image (from PIL).
        '''

        self._image_data['mime_type'] = self._filecontent.getContentType()
        PILInfo = Image.open(filename)
        self._image_data['width'],self._image_data['height'] = PILInfo.size


    _iptc_map = {
        'object name': 'title',
        # caption  will be used as Plone's description
        'caption/abstract': 'description', 
        'by-line': 'photographer',
        # Is Terms-of-Use available?
        'copyright notice': 'copyright',
        # keywords returns a python list of keywords
        'keywords': 'keywords', 
        # location would be something like "Boston Harbor"
        'sub-location': 'location', 
        'city': 'city',
        'province/state': 'state',
        'country/primary location name': 'country',
        # This is the ISO Country code 
        # (See http://userpage.chemie.fu-berlin.de/diverse/doc/ISO_3166.html)
        'country/primary location code': 'isoCountryCode'}

    _iptc_defaults = {'keywords': []}

    def _getIPTC(self, filename):
        """
        Get IPTC metadata information. This is typically data that is stored 
        in the image by various means, after it is created.
        """
        try:
            IPTCInfo = iptcinfo.IPTCInfo(filename).data
        except:
            IPTCInfo = {}

        for dataset in iptcinfo.c_datasets.itervalues():
            value = self._getValue(
                IPTCInfo, dataset,
                self._iptc_defaults.get(dataset, marker))
            if value is not marker:
                self._image_data[
                    self._iptc_map.get(dataset, dataset)] = value

    def _setIPTC(self, filename):
        """
        Set IPTC metadata information from the user's edits.
        """
        pass #for now

    def _getEXIFInfo(self, filename):
        file_ = open(filename)
        try:
            return EXIF.process_file(file_)
        except (ValueError, KeyError, TypeError, AttributeError):
            logger.exception(
                'EXIF processing failed for %s' % self._filecontent)
            return

    def _getEXIF(self, filename):
        """
        Get EXIF metadata information.  This is typically data that is set by
        the camera.
        """
        EXIFInfo = self._getEXIFInfo(filename)
        if EXIFInfo is None:
            return

# XXX: We should get other EXIF info that might be interesting to photographers...
#      See photo.net as example. Also, verify that exposureTime, focalLength, fNumber
#      are properly calculated.
#        if 'EXIF ExposureTime' in EXIFInfo:
#            self._image_data['exposureTime']    = EXIFInfo['EXIF ExposureTime'].printable
#        if 'EXIF FocalLength' in EXIFInfo:
#            self._image_data['focalLength']     = EXIFInfo['EXIF FocalLength'].printable
#        if 'EXIF Flash' in EXIFInfo:
#            self._image_data['flash']           = EXIFInfo['EXIF Flash'].printable.replace(' (!)','')
#        if 'EXIF FNumber' in EXIFInfo:
#            nom,denom = EXIFInfo['EXIF FNumber'].printable.split('/')
#            self._image_data['fNumber']         = 'f/%sL' % str( float(nom) / float(denom) )

        if 'EXIF DateTimeOriginal' in EXIFInfo:
            try:
                self._image_data[
                    'dateCreated'] = datetime.parseDatetimetz(
                    EXIFInfo['EXIF DateTimeOriginal'].printable)
            except datetime.DateTimeError:
                logger.exception(
                    'Invalid date in EXIF for %s' % self._filecontent)
        
        if 'MakerNote ISOSetting' in EXIFInfo:
            self._image_data['iso']             = 'ISO ' + str(eval(EXIFInfo['MakerNote ISOSetting'].printable)[1])
        if 'Image Model' in EXIFInfo:
            self._image_data['cameraModel']     = EXIFInfo['Image Model'].printable
        if 'Image ResolutionUnit' in EXIFInfo:
            self._image_data['resUnit']         = EXIFInfo['Image ResolutionUnit'].printable
        if 'Image XResolution' in EXIFInfo:
            self._image_data['xRes']            = EXIFInfo['Image XResolution'].printable
        if 'Image YResolution' in EXIFInfo:
            self._image_data['yRes']            = EXIFInfo['Image YResolution'].printable

        if 'GPS GPSLatitudeRef' in EXIFInfo:
            gpsLatRef = EXIFInfo['GPS GPSLatitudeRef'].values
        if 'GPS GPSLatitude' in EXIFInfo:
            xCoordDeg,xCoordMin,xCoordSec = EXIFInfo['GPS GPSLatitude'].values
            xCoordSecParts = str(xCoordSec).split('/')
            xCoordSecDeg = '%.5f' % (float(xCoordSecParts[0]) / float(xCoordSecParts[1]))
            self._image_data['gpsLat'] = '''%s Deg %s' %s" %s''' % (xCoordDeg,
                                                                 xCoordMin,
                                                                 xCoordSecDeg,
                                                                 gpsLatRef)

        if 'GPS GPSLongitudeRef' in EXIFInfo:
            gpsLongRef = EXIFInfo['GPS GPSLongitudeRef'].values
        if 'GPS GPSLongitude' in EXIFInfo:
            yCoordDeg,yCoordMin,yCoordSec = EXIFInfo['GPS GPSLongitude'].values
            yCoordSecParts = str(yCoordSec).split('/')
            yCoordSecDeg = '%.5f' % (float(yCoordSecParts[0]) / float(yCoordSecParts[1]))
            self._image_data['gpsLong'] = '''%s Deg %s' %s" %s''' % (yCoordDeg,
                                                                  yCoordMin,
                                                                  yCoordSecDeg,
                                                                  gpsLongRef)

        if 'GPS GPSAltitudeRef' in EXIFInfo:
            gpsAltRef = EXIFInfo['GPS GPSAltitudeRef'].values
        if 'GPS GPSAltitude' in EXIFInfo:
            zCoordDeg,zCoordMin,zCoordSec = EXIFInfo['GPS GPSAltitude'].values
            zCoordSecParts = str(zCoordSec).split('/')
            zCoordSecDeg = '%.5f' % (float(zCoordSecParts[0]) / float(zCoordSecParts[1]))
            self._image_data['gpsAlt'] = '''%s Deg %s' %s" %s''' % (zCoordDeg,
                                                                 zCoordMin,
                                                                 zCoordSecDeg,
                                                                 gpsAltRef)

        if 'GPS GPSMapDatum' in EXIFInfo:
            self._image_data['GPSDatum'] = EXIFInfo['GPS GPSMapDatum'].values



    def _setEXIF(self, filename):
        '''
        Set EXIF metadata information from the user's edits.
        '''
        pass # for now

    def _loadThumbnail(self, filename):
        '''
        Use thumbnails for images the same way we use embedded images in audio files.
        Retrieve the thumbnail from the image if it exists, otherwise make one.
        Note that this sort of functionality comes with ATImage, but since we also
        want to handle ATFile, we add it here.
        '''
        EXIFInfo = self._getEXIFInfo(filename)
        if EXIFInfo is None:
            return

        if 'JPEGThumbnail' in EXIFInfo:
            thumb = EXIFInfo['JPEGThumbnail']
            mime_type = 'image/jpeg'
        elif 'TIFFThumbnail' in EXIFInfo:
            thumb = EXIFInfo['TIFFThumbnail']
            mime_type = 'image/tiff'
        else:
            thumb = None

        if 'title' in self._image_data:
            try:
                thumb_title = self._image_data['title']+' thumbnail'
            except TypeError:
                thumb_title = 'Thumbnail'
        else:
            thumb_title = 'Thumbnail'

        # XXX When no thumbnail is available, create one using PIL. 
        #     Should be derived from original image and saved in JPEG format.

        if thumb is not None:
            ext = mimetypes.guess_extension(mime_type) or '.jpg'
            kwargs = dict(id=os.path.basename(filename)+ext, 
                          title=thumb_title, 
                          file=thumb,
                          content_type=mime_type)
            image = ofsimage.Image(**kwargs)
            self._image_data['image_thumbnail'] = image 

    def _storeThumbnail(self, filename):
        '''
        XXX do we want to be able to store a new thumbnail in the image?
        Audio code is:
        # saving the image(s)
        if self._image.image_thumbnail is not None:
            write_image_thumbnail(id3tags, self._image.image_thumbnail)
        We will replace write_image_thumbnail with this method...
        Or should this be a function, not a method like write_image_thumbnail is?
        Note EXIF can only handle jpeg or tiff thumbnails
        '''
        pass

    def load(self, filename):
        '''
        Depending on the mime type, load metadata by calling different functions.
        XXX load is called on initial create and on edit
            sync_image_metadata -> _load_image_metadata -> load
        XXX this won't work properly until store is working too
        '''

        self._getInfo(filename)

        if self._image_data['mime_type'] in ['image/jpeg','image/tiff']:
            self._getIPTC(filename)
            self._getEXIF(filename)
            self._loadThumbnail(filename)
        else:
            # Unknown image type. Pass for now.
            # XXX What will happen when gifs or some other types are uploaded?
            pass

    def store(self, filename):
        """
        Depending on the mime type, store metadata by calling different functions.
        XXX will we need to set the encoding for EXIF or IPTC?  if so it should
        XXX be UTF_8_ENCODING - relevant audio code:
          id3tags.setTextEncoding(eyeD3.frames.UTF_8_ENCODING)
          id3tags.setTitle(self._image.title or u'')
        """
        if self._image_data['mime_type'] in ['image/jpeg','image/tiff']:
            self._setIPTC(filename)
            self._setEXIF(filename)
            #self._storeThumbnail(filename) XXX is this desireable?
        else:
            # Unknown image type. Pass for now.
            # XXX What will happen when gifs or some other types are uploaded?
            pass



DEFAULT_CHARSET = 'utf-8'

def unicodestr(v, charset=DEFAULT_CHARSET):
    """Return the unicode object representing the value passed in an
    as error-immune manner as possible.
    
      >>> unicodestr(u'foo')
      u'foo'
      >>> unicodestr('bar')
      u'bar'
      >>> unicodestr('héllo wórld', 'ascii')
      u'h\\ufffd\\ufffdllo w\\ufffd\\ufffdrld'
    """

    if isinstance(v, unicode):
        return v
    if isinstance(v, str):
        return v.decode(charset, 'replace')
    return unicode(v)



from zope import interface
from zope import schema

from p4a.fileimage import file as p4afile
from p4a.fileimage import image as p4aimage
#from p4a.image import genre  # don't think we want Genre

try:
    from zope.dublincore.interfaces import IDCDescriptiveProperties
except ImportError:
    from zope.app.dublincore.interfaces import IDCDescriptiveProperties
    

from zope.i18nmessageid import MessageFactory
_ = MessageFactory('plone4artists')

class IAnyImageCapable(interface.Interface):
    """
    Any aspect of image/content capable.
    """


class IPossibleImage(IAnyImageCapable):
    """
    All objects that should have the ability to be converted to some
    form of image should implement this interface.
    """


class IImageEnhanced(interface.Interface):
    """
    All objects that have their media features activated/enhanced
    should have this marker interface applied.
    """


class IImage(interface.Interface):
    """
    Objects which have image information.
    """
    title = schema.TextLine(
        title=_(u'Image Title'), required=False)
    description = schema.Text(
        title=_(u'Caption'), required=False)
    file = p4afile.FileField(
        title=_(u'File'), required=False)
    # XXX: Really need to get rid of the preferred dimensions in image_thumbnail! This is a remnant 
    #      of p4a.audio. This will cause problems with the aspect ration of the thumbnails!
    image_thumbnail = p4aimage.ImageField(
        title=_(u'Image Thumbnail'), required=False,
        preferred_dimensions=(400, 300))
    photographer = schema.TextLine(
        title=_(u'Photographer'), required=False)
    dateCreated = schema.Datetime(
        title=_(u'Date Created'), required=False)
    copyright = schema.TextLine(
        title=_(u'Copyright'), required=False)
    keywords = schema.List(
        title=_(u'Keywords'), required=False,
        default=[], missing_value=[],
        description=_(u'Enter a commas separated list of keywords'),
        value_type=schema.TextLine())
    location = schema.TextLine(
        title=_(u'Location'), required=False)
    city = schema.TextLine(
        title=_(u'City'), required=False)
    state = schema.TextLine(
        title=_(u'Province or State'), required=False)
    country = schema.TextLine(
        title=_(u'Country'), required=False)
    resUnit = schema.ASCIILine(
        title=_(u'Resolution Unit'), required=False, readonly=True)
    xRes = schema.ASCIILine(
        title=_(u'Horizontal Resolution (dpi)'), required=False,
        readonly=True)
    yRes = schema.ASCIILine(
        title=_(u'Vertical Resolution (dpi)'), required=False,
        readonly=True)
    comment = schema.Text(
        title=_(u'Comment'), required=False)
    cameraModel = schema.ASCIILine(
        title=_(u'Camera Model'), required=False, readonly=True)
    iso = schema.ASCIILine(
        title=_(u'ISO'), required=False, readonly=True)
    width = schema.Int(
        title=_(u'Width'), required=True, readonly=True)
    height = schema.Int(
        title=_(u'Height'), required=True, readonly=True)
    image_type = schema.ASCIILine(
        title=_(u'Image Type'), required=True, readonly=True)
    gpsLat = schema.ASCIILine(
        title=_(u'GPS Latitude'), required=False)
    gpsLong = schema.ASCIILine(
        title=_(u'GPS Longitude'), required=False)
    gpsAlt = schema.TextLine(
        title=_(u'GPS Altitude'), required=False)


class IImageDataAccessor(interface.Interface):
    """
    Image implementation accessor (ie jpeg, tiff, etc).
    """

    image_type = schema.TextLine(title=_(u'Image Type'), required=True, readonly=True)
    
    def load(filename):
        pass
    def store(filename):
        pass


class IPossibleImageContainer(IAnyImageCapable):
    """
    Any folderish entity that can be turned into an actual image 
    container.
    """


class IImageContainerEnhanced(interface.Interface):
    """
    Any folderish entity that has had it's IImageContainer features
    enabled.
    """


class IImageContainer(interface.Interface):
    """
    Folderish objects that have image information, typically representing an 
    image collection or slideshow."""
    
    title           = schema.TextLine(title=_(u'Title'), required=False)
    description     = schema.Text(title=_(u'Description'), required=False)
    photographer    = schema.TextLine(title=_(u'Photographer'), required=False)
    copyright       = schema.TextLine(title=_(u'Copyright'), required=False)
    # XXX what to do about image here?
    folder_image    = p4aimage.ImageField(title=_(u'Folder Image'),required=False,preferred_dimensions=(400, 300))
    dateCreated     = schema.TextLine(title=_(u'Date Created'), required=False)


class IImageProvider(interface.Interface):
    """
    Provide image.
    """
    
    image_items = schema.List(title=_(u'Image Items'),required=True,readonly=True)


class IBasicImageSupport(interface.Interface):
    """
    Provides certain information about image support.
    """

    support_enabled = schema.Bool(title=u'Image Support Enabled?',
                                  required=True,
                                  readonly=True)


class IImageSupport(IBasicImageSupport):
    """
    Provides full information about image support.
    """


class IMediaActivator(interface.Interface):
    """For seeing the activation status or toggling activation."""

    media_activated = schema.Bool(title=u'Media Activated',
                                  required=True,
                                  readonly=False)


# Make the image geospatially aware
# From http://svn.gispython.org/zope/zgeo.geographer/trunk/zgeo/geographer/interfaces.py
# XXX Do we want more? (IGeoCollectionSimple; IGeoreferenceable)
# Sean Gillies: "ideally, anything that implements IGeoItemSimple can be output
#   to desktop GIS, or served up as GeoRSS/KML for the web"

class IGeometry(interface.Interface):

    """
    A geometry property with a geographic or projected coordinate system.

    The spatial reference system is implicitly lat/long WGS84, aka EPSG:4326.
    """

    # TODO: geometries other than points
    type = schema.Choice(
        values=("Point",),  # Eventually add LineString, Polygon, etc.
        title=u"Geometry Type",
        description=u"The name of the geometry type. See "
        "http://code.google.com/apis/kml/documentation/kml_tags_21.html",
        required=False,
        default="Point",
        )

    coordinates = schema.Tuple(
        title=u"Geometry Coordinates",
        description=u"A sequence of coordinate tuples",
        value_type=schema.Tuple(title=u"Coordinates"),
        required=False,
        default=(),
        )


class IGeoItemSimple(IDCDescriptiveProperties):
    """
    A simple georeferenced object, analogous to an item in GeoRSS, or a
    KML placemark.

    See http://georss.org/simple.html for an explanation of the simple GeoRSS
    profile.

    IDCDescriptiveProperties defines 'title' and 'description'.
    """

    id = schema.Id(
        title=u"Identifier",
        description=u"Unique identifer for the item",
        required=True,
        )

    uri = schema.URI(
        title=u"URI",
        description=u"Uniform Resource Identifier",
        required=False,
        )

    geometry = schema.Object(
        IGeometry,
        title=u"Geometry",
        description=u"The item's geometry",
        required=False,
        default=None,
        )

    info = schema.Dict(
        title=u"Info Dictionary",
        description=u"Dictionary which provides the Python feature protocol",
        readonly=True,
        )



import urllib

from zope import event
from zope import interface
from zope.formlib import form
from zope.app.form import browser as za_form_browser
from zope.cachedescriptors import property
from zope.app.event import objectevent
from zope.i18n import translate
from zope.i18nmessageid import MessageFactory

from Products.CMFCore import utils as cmfutils
from Products.statusmessages import interfaces as statusmessages_ifaces

from p4a.common import formatting
from p4a.fileimage.image._widget import ImageURLWidget

#from p4a.image import genre # XXX are we going to use?
from p4a.image import interfaces
from p4a.image.browser import media

_ = MessageFactory('plone4artists')

# XXX: IImageView needs to be modified to fit what is appropriate for images.
# XXX: Rocky says this module should be seriously refactored...

class IImageView(interface.Interface):
    """
    """
    def title():        pass


class ImageView(object):
    """
    """

    def __init__(self, context, request):
        self.image_info = interfaces.IImage(context)

        mime_type = unicode(context.get_content_type())

    def image_type(self):   return self.image_info.image_type
    # Use the title from metadata as plone title if it exists
    def title(self):        return self.image_info.title
    # Likewise for plone description (called caption in metadata)
    def description(self):  return self.image_info.description
    def photographer(self): return self.image_info.photographer
    def width(self):        return self.image_info.width
    def height(self):       return self.image_info.height
    def copyright(self):    return self.image_info.copyright
    def keywords(self):     return self.image_info.keywords
    def location(self):     return self.image_info.location
    def city(self):         return self.image_info.city
    def state(self):        return self.image_info.state
    def country(self):      return self.image_info.country
    def resUnit(self):      return self.image_info.resUnit
    def xRes(self):         return self.image_info.xRes
    def yRes(self):         return self.image_info.yRes
    def comment(self):      return self.image_info.comment
    def cameraModel(self):  return self.image_info.cameraModel
    def iso(self):          return self.image_info.iso
    # DMS => Degrees Minutes Seconds
    def gpsDMSLat(self):    return self.image_info.gpsLat
    def gpsDMSLong(self):   return self.image_info.gpsLong
    def gpsDMSAlt(self):    return self.image_info.gpsAlt

    def dateCreated(self):
        date = self.image_info.dateCreated
        if date is not None:
            return date.strftime('%d %B %Y - %H:%M')
        return ''

    # Also, produce the gps coords as degrees for easy linking
    # to these coordinates in Google Maps, Yahoo Maps, etc.
    def gpsDegLatStr(self):
        if self.image_info.gpsLat:
            latParts     = self.image_info.gpsLat.replace(' Deg','').replace('"','').replace("'",'').split(' ')
            negPos = 1
            if latParts[3] == 'S':
                negPos = -1
            gpsDegLatStr = '%.6f' % (negPos * ((float(latParts[0]) / 1) + (float(latParts[1]) / 60) + (float(latParts[2]) / 3600)))
            return gpsDegLatStr
        else:
            return None

    def gpsDegLongStr(self):
        if self.image_info.gpsLong:
            longParts     = self.image_info.gpsLong.replace(' Deg','').replace('"','').replace("'",'').split(' ')
            negPos = 1
            if longParts[3] == 'W':
                negPos = -1
            gpsDegLongStr = '%.6f' % (negPos * ((float(longParts[0]) / 1) + (float(longParts[1]) / 60) + (float(longParts[2]) / 3600)))
            return gpsDegLongStr
        else:
            return None


class ImagePageView(media.BaseMediaDisplayView):
    """
    Page for displaying image.
    """

    adapted_interface = interfaces.IImage
    media_field = 'file'

    form_fields = form.FormFields(interfaces.IImage)
    label = u'View Image Info'

    @property.Lazy
    def template(self):
        return self.index

    def has_contentlicensing_support(self):
        try:
            from Products import ContentLicensing
            ContentLicensing # pyflakes
        except ImportError, e:
            return False

        try:
            cmfutils.getToolByName(self.context, 'portal_contentlicensing')
        except AttributeError, e:
            return False

        return True


def applyChanges(context, form_fields, data, adapters=None):
    if adapters is None:
        adapters = {}

    changed = []

    for form_field in form_fields:
        field = form_field.field
        # Adapt context, if necessary
        interface = field.interface
        adapter = adapters.get(interface)
        if adapter is None:
            if interface is None:
                adapter = context
            else:
                adapter = interface(context)
            adapters[interface] = adapter

        name = form_field.__name__
        newvalue = data.get(name, form_field) # using form_field as marker
        if (newvalue is not form_field) and (field.get(adapter) != newvalue):
            changed.append(name)
            field.set(adapter, newvalue)

    return changed

class KeywordWidget(za_form_browser.TextWidget):
    """Split the value into a list by commas"""

    def _toFieldValue(self, input):
        value = super(KeywordWidget, self)._toFieldValue(input)
        if self.convert_missing_value and input == self._missing:
            return value
        return [i.strip() for i in value.split(',') if i.strip()]

    def _toFormValue(self, value):
        if value != self.context.missing_value:
            value = ', '.join(value)
        return super(KeywordWidget, self)._toFormValue(value)

class ImageEditForm(form.EditForm):
    """
    Form for editing image fields.
    """

    form_fields = form.FormFields(interfaces.IImage)
    form_fields['keywords'].custom_widget = KeywordWidget
    label = _(u'Edit Image Data')
    
    @form.action(_(u"Apply"), condition=form.haveInputWidgets)
    def handle_edit_action(self, action, data):
        changed = applyChanges(
            self.context, self.form_fields, data, self.adapters)
        if changed:
            attrs = objectevent.Attributes(interfaces.IImage, *changed)
            event.notify(
                objectevent.ObjectModifiedEvent(self.context, attrs)
                )
            # TODO: Needs locale support. See also Five.form.EditView.
            self.status = _(u"Successfully updated")
        else:
            self.status = _(u'No changes')
        statusmessages_ifaces.IStatusMessage(
            self.request).addStatusMessage(self.status, 'info')
        redirect = self.request.response.redirect
        return redirect(self.context.absolute_url()+'/view')


class ImageContainerEditForm(form.EditForm):
    """
    Form for editing image container fields.
    """

    form_fields = form.FormFields(interfaces.IImageContainer)
    label = _(u'Edit Image Container Data')

    @form.action(_(u"Apply"), condition=form.haveInputWidgets)
    def handle_edit_action(self, action, data):
        changed = applyChanges(
            self.context, self.form_fields, data, self.adapters)
        if changed:
            attrs = objectevent.Attributes(interfaces.IImageContainer, *changed)
            event.notify(
                objectevent.ObjectModifiedEvent(self.context, attrs)
                )
            # TODO: Needs locale support. See also Five.form.EditView.
            self.status = _(u"Successfully updated")
        else:
            self.status = _(u'No changes')
        redirect = self.request.response.redirect
        msg = urllib.quote(translate(self.status))
        redirect(self.context.absolute_url()+'/view?portal_status_message=%s' % msg)


class ImageContainerView(form.PageDisplayForm):
    """
    View for image containers.
    """
    adapted_interface = interfaces.IImageContainer

    form_fields = form.FormFields(interfaces.IImageContainer)
    label = _(u'View Image Container Info')

    @property.Lazy
    def template(self):
        return self.index

    @property.Lazy
    def image_info(self):
        return interfaces.IImageContainer(self.context)

    @property.Lazy
    def _image_items(self):
        provider = interfaces.IImageProvider(self.context)
        # cheating here by getting image properties we need by looking
        # up context attribute which isn't in the interface contract
        results = []
        for x in provider.image_items:
            aImage = x.context
            field = aImage.getImage()
            results.append(
                {'title': x.title,
                 'description': x.context.Description(),
                 'photographer': x.photographer,
                 'size': formatting.fancy_data_size(field.get_size()),
                 'icon': aImage.getIcon(),
                 'url': aImage.absolute_url()})
        return results
        
    def _imageurlwidget(self, image_item):
        field = interfaces.IImage['image_thumbnail'].bind(image_item)
        w = ImageURLWidget(field, self.request)
        return w()

    def image_items(self):
        return self._image_items
    
    def photographer(self):
        return self.image_info.photographer
        
    def dateCreated(self):
        return self.image_info.dateCreated

    def has_syndication(self):
        try:
            view = self.context.restrictedTraverse('@@rss.xml')
            return True
        except:
            # it's ok if this doesn't exist, just means no syndication
            return False

    def should_display_summary(self):
        #XXX add this if using genre:   or self.image_info.genre \
        return self.image_info.photographer \
               or self.image_info.dateCreated \
               or len(self._image_items) > 0

    def should_display_art(self):
        return self.image_info.image_thumbnail is not None



import unittest
from zope.component import testing
from zope.testing import doctestunit

def test_suite():
    return unittest.TestSuite((
        doctestunit.DocTestSuite('p4a.image.utils'),
        doctestunit.DocFileSuite('migration.txt',
                                 package="p4a.image",
                                 setUp=testing.setUp,
                                 tearDown=testing.tearDown),
        doctestunit.DocFileSuite('extraction.txt',
                                 setUp=testing.setUp,
                                 tearDown=testing.tearDown)
        ))



# XXX - Do we need this file ??????


from zope import component
from zope.app.form.browser import widget
from p4a.image import interfaces
from p4a.fileimage import file

class MediaPlayerWidget(file.FileDownloadWidget):
    """Widget which produces some form of media player.
    """
    
    def __call__(self):
        file_present = True
        if not self._data:
            file_present = False
        
        url = self.url
        if not url:
            file_present = False
        
        if not file_present:
            return widget.renderElement(u'span',
                                        cssClass='media-absent media-player',
                                        contents='No media to play')
        
        field = self.context
        contentobj = field.context.context

        mime_type = unicode(contentobj.get_content_type())
        media_player = component.queryAdapter(field,
                                              interface=interfaces.IMediaPlayer,
                                              name=mime_type)
        
        if media_player is None:
            return widget.renderElement(u'span',
                                        cssClass='player-not-available media-player',
                                        contents='No available player for mime type "%s"' % mime_type)
        
        return u"""<div class="media-player">%s</div>""" % media_player(url)



from zope.formlib import form
from p4a.image import interfaces
from p4a.image.browser import widget

class ToggleEnhancementsView(object):
    """
    """

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        was_activated = self.media_activated
        self.media_activated = not was_activated
        response = self.request.response
        
        if was_activated:
            activated = 'Media+deactivated'
        else:
            activated = 'Media+activated'
        
        response.redirect(self.context.absolute_url()+'/view?portal_status_message='+activated)

    def _set_media_activated(self, v):
        interfaces.IMediaActivator(self.context).media_activated = v
    def _get_media_activated(self):
        return interfaces.IMediaActivator(self.context).media_activated
    media_activated = property(_get_media_activated, _set_media_activated)

class BaseMediaDisplayView(form.PageDisplayForm):
    """
    Base view for displaying media.
    """

    adapted_interface = None
    media_field = None

    def _media_player(self):
        image = self.adapters.get(self.adapted_interface,
                                  self.adapted_interface(self.context))
        field = self.adapted_interface[self.media_field].bind(image)
        player_widget = widget.MediaPlayerWidget(field, self.request)
        player_widget.name = self.prefix + 'media_player'
        player_widget._data = field.get(image)
        return player_widget

    def update(self):
        super(BaseMediaDisplayView, self).update()
        player_widget = self._media_player()
        self.widgets += form.Widgets([(None, player_widget)], len(self.prefix))



from setuptools import setup, find_packages

version = '0.1'

readme = open('README.txt')
long_description = readme.read()
readme.close()

setup(name='p4a.image',
      version=version,
      description="",
      long_description=long_description,
      classifiers=[], # Get strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      keywords='',
      author='David Siedband',
      author_email='technique@oceanicsky.com',
      url='http://svn.plone.org/svn/plone/plone.example',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['p4a'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          # -*- Extra requirements: -*-
          'p4a.subtyper',
          'p4a.z2utils',
          'p4a.fileimage',
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



from zope import component
from zope import interface
from zope import schema
from p4a.image import interfaces

class IContextualImageSupport(interfaces.IBasicImageSupport):
    can_activate_image = schema.Bool(title=u'Can Activate Image',
                                     readonly=True)
    can_deactivate_image = schema.Bool(title=u'Can Deactivate Image',
                                       readonly=True)

class Support(object):
    """
    A view that returns certain information regarding p4acal status.
    """

    interface.implements(IContextualImageSupport)

    def __init__(self, context, request):
        self.context = context
        self.request = request
        
    @property
    def support_enabled(self):
        """
        Check to make sure an IImageSupport utility is available and
        if so, query it to determine if support is enabled.
        """
        
        support = component.queryUtility(interfaces.IImageSupport)
        if support is None:
            return False

        return support.support_enabled

    @property
    def _basic_can(self):
        if not self.support_enabled:
            return False

        if not interfaces.IAnyImageCapable.providedBy(self.context):
            return False

        return True

    @property
    def can_activate_image(self):
        if not self._basic_can:
            return False
        
        mediaconfig = component.getMultiAdapter((self.context, self.request),
                                                name='media-config.html')
        return not mediaconfig.media_activated

    @property
    def can_deactivate_image(self):
        if not self._basic_can:
            return False
        
        mediaconfig = component.getMultiAdapter((self.context, self.request),
                                                name='media-config.html')
        return mediaconfig.media_activated
